"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = require("ava");
const index_1 = require("./index");
ava_1.default("find with normal pattern", (t) => {
    const result = index_1.find("bonjour", "on");
    t.deepEqual(result, [2, 3]);
});
ava_1.default("find with normal pattern but don't find", (t) => {
    const result = index_1.find("bonjour", "zn");
    t.deepEqual(result, []);
});
ava_1.default("find with complex pattern", (t) => {
    const result = index_1.find("bonjour", "^[%a_][%w_]*[.:]?[%w_.]*");
    t.deepEqual(result, [1, 7]);
});
ava_1.default("find with complex pattern, no result", (t) => {
    const result = index_1.find(",test", "^[%a_][%w_]*[.:]?[%w_.]*");
    t.deepEqual(result, []);
});
ava_1.default("find with pattern", (t) => {
    const result = index_1.find("bonjour", "o%w");
    t.deepEqual(result, [2, 3]);
});
ava_1.default("find with back reference", (t) => {
    const result = index_1.find('""', "^((['\"])%2)");
    t.deepEqual(result, [1, 2]);
});
ava_1.default("find with back reference 2", (t) => {
    const result = index_1.find('"test"', `^(['"]).-[^\\]%1`);
    t.deepEqual(result, [1, 6]);
});
ava_1.default("gmatch", (t) => {
    const result = index_1.gmatch("a text with words", "%w+");
    t.not(result, undefined);
    if (result == undefined)
        return;
    const results = [];
    for (;;) {
        const value = result();
        if (!value)
            break;
        results.push(value);
    }
    t.deepEqual(results, ["a", "text", "with", "words"]);
});
ava_1.default("gsub", (t) => {
    const result = index_1.gsub("a text with words", "w", "v");
    t.is(result, "a text vith vords");
});
ava_1.default("gsub with captures", (t) => {
    const result = index_1.gsub("a text with words", "w(%w)", "%1v");
    t.is(result, "a text ivth ovrds");
});
ava_1.default("gsub with captures at start", (t) => {
    const result = index_1.gsub("hello world", "(%w+)", "%1 %1");
    t.is(result, "hello hello world world");
});
ava_1.default("gsub with captures %0", (t) => {
    const result = index_1.gsub("hello world", "%w+", "%0 %0");
    t.is(result, "hello hello world world");
});
ava_1.default("gsub with two captures", (t) => {
    const result = index_1.gsub("hello world from Lua", "(%w+)%s*(%w+)", "%2 %1");
    t.is(result, "world hello Lua from");
});
ava_1.default.failing("gsub with function", (t) => {
    const result = index_1.gsub("4+5 = $return 4+5$", "%$(.-)%$", (s) => {
        return eval(s);
    });
    t.is(result, "4+5 = 9");
});
ava_1.default("format", (t) => {
    const result = index_1.format("un %s truc %d", "azz", 12);
    t.is(result, "un azz truc 12");
});
ava_1.default("format with %%", (t) => {
    const result = index_1.format("un %s %%w truc %d", "azz", 12);
    t.is(result, "un azz %w truc 12");
});
ava_1.default("gmatch with line endings", (t) => {
    const result = [];
    const iterable = index_1.gmatch("first\r\nsecond", "[^\r\n]+");
    if (iterable === undefined) {
        t.fail();
        return;
    }
    for (const m of iterable) {
        result.push(m);
    }
    if (result === null) {
        t.fail();
        return;
    }
    t.is(result[0], "first");
    t.is(result[1], "second");
});
ava_1.default("match that removes spaces", (t) => {
    const result = index_1.match(`deathknight="Death_Knight_Frost_T19P"`, "^%s*(.-)%s*$");
    if (result === null) {
        t.fail();
        return;
    }
    t.is(result[0], `deathknight="Death_Knight_Frost_T19P"`);
});
ava_1.default("match that doesn't match", (t) => {
    const result = index_1.match("actions", `^actions%.([%w_]+)`);
    t.is(result.length, 0);
});
ava_1.default("sub with positive end", (t) => {
    const result = index_1.sub("teste", 2, 4);
    t.is(result, "est");
});
ava_1.default("sub with negative end", (t) => {
    const result = index_1.sub("teste", 2, -2);
    t.is(result, "est");
});
// test("gsub with function", t => {
//     const result = gsub("a text with words", "w(%w)", (s, t) => t + s);
//     t.is(result, "a text ivth ovrds");
// });
//# sourceMappingURL=index.spec.js.map